Exception: PGN::UnconsumedInputError
- Inherits:
-
StandardError
- Object
- StandardError
- PGN::UnconsumedInputError
- Defined in:
- lib/pgn/lexer.rb
Overview
Raised when the lexer cannot match the input at the current position.
Raised when the lexer cannot match the input at the current position.